@@ -3,7 +3,7 @@ from __future__ import absolute_import
 from builtins import str
 import logging
 from . import utils
-from .utils import safe_read_simple_url, safe_get_json
+from .utils import safe_read_simple_url, safe_get_json, safe_put_simple_url
 import os
 
 """
@@ -20,8 +20,10 @@ def getAwsInstanceDetails(config, authentication_token, account_id, host, log, d
                             'spectrum_dimensions': ['instance_id'],
                             'authentication_token': 'DUMMY-TOKEN',
                             'host': 'host-123'}
+
+    metadata_token_url = config.get('aws_metadata_service_token_url', None)    
     locator_url = config['aws_instance_id_locator_url']
-    instance_id = getAwsInstanceId(locator_url, log)
+    instance_id = getAwsInstanceId(locator_url, log, metadata_token_url=metadata_token_url)
     if instance_id and authentication_token:
         instance_details = {'cloud_provider': 'AWS', 'instance_id': instance_id,
                             'authentication_token': authentication_token,
@@ -125,10 +127,8 @@ def init__configuration(config, cloud_provider, debug=False):
     creds = utils.retrieve_creds()
     authentication_token = creds["token"]
     account_id = creds["account_id"]
-    # if the user supply custom host we take it from env variables
-    # else take the default from config file(spotinst-agent.yml)
-    host = os.getenv('CUSTOM_HOST', config['default_host'])
-
+    host = creds["host"]
+    
     if cloud_provider.lower() == 'azure':
         instance_details = getAzureInstanceDetails(config, authentication_token, account_id, host, log, debug)
     elif cloud_provider.lower() == 'azure_spot':
@@ -140,11 +140,25 @@ def init__configuration(config, cloud_provider, debug=False):
     return instance_details
 
 
-def getAwsInstanceId(locator_url, log):
+def getAwsInstanceId(locator_url, log, metadata_token_url=None):
     response = None
     if locator_url:
         if locator_url.startswith("http://"):
-            response = safe_read_simple_url(locator_url, log)
+            aws_token = None
+
+            if metadata_token_url:
+                aws_token = _fetch_aws_metadata_token(metadata_token_url, log=log)
+            else:
+                log.error("Url for fetching aws metadata token doen't exists in config")
+
+            headers = {}
+            
+            if aws_token:
+                headers['x-aws-ec2-metadata-token'] = aws_token
+            else:
+                log.warn("Failed to retrieve aws token, using IMDSv1 instead")
+
+            response = safe_read_simple_url(locator_url, log, headers=headers)
             response = str(response)
 
         else:
@@ -159,6 +173,20 @@ def getAwsInstanceId(locator_url, log):
     return response
 
 
+def _fetch_aws_metadata_token(token_url, log=logging.getLogger('spotinst-agent'), token_ttl=21600):
+    response = None
+    headers = {
+        'x-aws-ec2-metadata-token-ttl-seconds': str(token_ttl),
+    }
+
+    token_response = safe_put_simple_url(token_url, log, headers)
+
+    if token_response:
+        response = str(token_response)
+
+    return response
+
+
 def getGcpInstanceMetadata(locator_url, path_params, log):
     response = None
     if locator_url:
